// Copyright (C) 2022 Rob Caelers <robc@krandor.nl>
// All rights reserved.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//

#ifndef WORKRAVE_UI_PREFWIDGETS_CHOICE_HH
#define WORKRAVE_UI_PREFWIDGETS_CHOICE_HH

#include <functional>
#include <vector>

#include "Widget.hh"

namespace ui::prefwidgets
{
  class Choice : public WidgetBase<Choice, int>
  {
  public:
    Choice() = default;
    explicit Choice(const std::string &label);
    ~Choice() override = default;

    static std::shared_ptr<Choice> create();
    static std::shared_ptr<Choice> create(const std::string &label);

    std::shared_ptr<Choice> assign(const std::vector<std::string> &options);
    std::shared_ptr<Choice> assign(std::function<std::vector<std::string>()> func);
    std::vector<std::string> get_content() const;

  private:
    std::vector<std::string> content;
    std::function<std::vector<std::string>()> fill_func;
  };
} // namespace ui::prefwidgets

#endif // WORKRAVE_UI_PREFWIDGETS_CHOICE_HH
